@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2021 Joel Rosdahl and other contributors
+// Copyright (C) 2020-2022 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -42,13 +42,7 @@ ResultRetriever::ResultRetriever(Context& ctx, bool rewrite_dependency_target)
 }
 
 void
-ResultRetriever::on_header(core::CacheEntryReader& /*cache_entry_reader*/,
-                           const uint8_t /*result_format_version*/)
-{
-}
-
-void
-ResultRetriever::on_entry_start(uint32_t entry_number,
+ResultRetriever::on_entry_start(uint8_t entry_number,
                                 FileType file_type,
                                 uint64_t file_len,
                                 nonstd::optional<std::string> raw_file)
@@ -76,9 +70,10 @@ ResultRetriever::on_entry_start(uint32_t entry_number,
     }
     break;
 
+  case FileType::stdout_output:
   case FileType::stderr_output:
-    // Stderr data: Don't open a destination file. Instead accumulate it in
-    // m_dest_data and write it in on_entry_end.
+    // Stdout/stderr data: Don't open a destination file. Instead accumulate it
+    // in m_dest_data and write it in on_entry_end.
     m_dest_data.reserve(file_len);
     break;
 
@@ -114,7 +109,8 @@ ResultRetriever::on_entry_start(uint32_t entry_number,
     break;
   }
 
-  if (file_type == FileType::stderr_output) {
+  if (file_type == FileType::stdout_output
+      || file_type == FileType::stderr_output) {
     // Written in on_entry_end.
   } else if (dest_path.empty()) {
     LOG_RAW("Not writing");
@@ -141,9 +137,12 @@ ResultRetriever::on_entry_start(uint32_t entry_number,
 void
 ResultRetriever::on_entry_data(const uint8_t* data, size_t size)
 {
-  ASSERT(!(m_dest_file_type == FileType::stderr_output && m_dest_fd));
+  ASSERT(!((m_dest_file_type == FileType::stdout_output
+            || m_dest_file_type == FileType::stderr_output)
+           && m_dest_fd));
 
-  if (m_dest_file_type == FileType::stderr_output
+  if (m_dest_file_type == FileType::stdout_output
+      || m_dest_file_type == FileType::stderr_output
       || (m_dest_file_type == FileType::dependency && !m_dest_path.empty())) {
     m_dest_data.append(reinterpret_cast<const char*>(data), size);
   } else if (m_dest_fd) {
@@ -158,9 +157,12 @@ ResultRetriever::on_entry_data(const uint8_t* data, size_t size)
 void
 ResultRetriever::on_entry_end()
 {
-  if (m_dest_file_type == FileType::stderr_output) {
+  if (m_dest_file_type == FileType::stdout_output) {
+    LOG("Writing to file descriptor {}", STDOUT_FILENO);
+    Util::send_to_fd(m_ctx, m_dest_data, STDOUT_FILENO);
+  } else if (m_dest_file_type == FileType::stderr_output) {
     LOG("Writing to file descriptor {}", STDERR_FILENO);
-    Util::send_to_stderr(m_ctx, m_dest_data);
+    Util::send_to_fd(m_ctx, m_dest_data, STDERR_FILENO);
   } else if (m_dest_file_type == FileType::dependency && !m_dest_path.empty()) {
     write_dependency_file();
   }
